from microboiler.modules.basemodule import BaseModule
from microboiler.modules.templating.templating import replace_template_file
import os
import yaml
class Docker(BaseModule):
    # Docker singleton Instance.
    __instance = None
    dockerOptions = {'version' : "3", 'services': {},'volumes':{} ,'networks':{'localnet':{'driver':'bridge'}}}
    @staticmethod
    def getInstance():
        """ Static access method. """
        return Docker.__instance 
    @staticmethod
    def initSingleton(projectOptions, project_templates_paths, outputPath):
        if Docker.__instance == None:
            Docker(projectOptions, project_templates_paths, outputPath)

    def __init__(self,projectOptions, project_templates_paths, outputPath):
        """ Virtually private constructor. """
        if Docker.__instance != None:
            raise Exception("This class is a singleton!")
        else:
            Docker.__instance = self
            super().__init__(projectOptions, project_templates_paths, outputPath)
    def AddService(self,name,options):
        self.dockerOptions['services'][name] = options   
    def AddVolume(self,name):
        self.dockerOptions['volumes'][name] = {}        
    def DumpTo(self, file):
        with open(file, 'w') as yaml_file:
            yaml.dump(self.dockerOptions, yaml_file, default_flow_style=False)
    def DockerComposeFinalization(self, file):
        replace_dict = {
            'rabbitmq:healtcheck': '["CMD", "curl", "-f", "http://localhost:15672"]'
        }
        replace_template_file(file,replace_dict)
    



from microboiler.modules.basemodule import BaseModule
from microboiler.modules.devops.docker import Docker
import os
class Mysql(BaseModule):
    def __init__(self, projectOptions, project_templates_paths, outputPath):
        super().__init__(projectOptions, project_templates_paths, outputPath)

    def HandleMysql(self, db_options):
        docker_options = self.BuildDockerOptions(db_options)
        docker_instance = Docker.getInstance()
        docker_instance.AddService(db_options['name'],docker_options)
        docker_instance.AddVolume('mysql-volume')

    def BuildDockerOptions(self, db_options):
        docker_volume_dir = os.path.normpath(os.path.join(self.outputPath,'docker_volumes','mysql'))
        if not os.path.isdir(docker_volume_dir):
            os.makedirs(docker_volume_dir)
        default_mysql_options = {

            'image': 'mysql/mysql-server:5.7',
            'container_name': db_options['name'],
            'command': 'mysqld --user=root --verbose',
            'volumes': ['mysql-volume:'+docker_volume_dir],
            'networks':['localnet'],            
            'ports': ['3306:3306'],
            'environment': {
                'MYSQL_ROOT_HOST': "%",
                'MYSQL_ALLOW_EMPTY_PASSWORD': "false"
            }          
        }

        # Set username and password
        if 'username' in db_options:
            default_mysql_options['environment']['MYSQL_USER'] = db_options['username']
        if 'password' in db_options:
            default_mysql_options['environment']['MYSQL_PASSWORD'] = db_options['password']
            default_mysql_options['environment']['MYSQL_ROOT_PASSWORD'] = db_options['password']

        if 'docker_compose_override' in db_options:
            default_mysql_options.update(db_options['docker_compose_override'])
        return default_mysql_options

    
def BuildMysqlConnectionString(server_host,database_name,user,password):       
    conn_string ="Server={0};Database={1};Username={2};Password={3}".format(server_host,database_name,user,password)
    conn_string_dev = "Server={0};Database={1};Username={2};Password={3}".format('localhost',database_name,user,password)
    return conn_string, conn_string_dev



from microboiler.modules.basemodule import BaseModule
from microboiler.modules.templating.templating import filter_region,filter_region_with_tag,filter_sub_region, replace_template_file
from microboiler.modules.utils.utils import FindDatabaseWithName, FindEventBusWithName, FindServerWithName
import os
class Csharp(BaseModule):
    def __init(self,  projectOptions, project_templates_paths, outputPath, client_options):
        super().__init__()
    def HandleCsprojLogging(self, service, host_csproj_path):
        logging_enabled = 'logging' in service
        if logging_enabled:
            if 'type' not in service['logging']:
                logging_type = 'serilog'
            else:
                logging_type = 'microsoft'
        if(logging_enabled):
            filter_sub_region(host_csproj_path,'logging',logging_type)
        else:
            filter_region_with_tag(host_csproj_path,'logging')
    def HandleCsprojDatabase(self, service_options, host_csproj_path):
        database_enabled = 'database' in service_options        
        if(database_enabled):
            database_instance = FindDatabaseWithName(self.projectOptions, service_options['database']['provider'])
            if database_instance is None:
                print ('Could not found database with name'+service_options['database']['provider'])
            database_type = database_instance['type']
            filter_sub_region(host_csproj_path,'database',database_type)
        else:
            filter_region_with_tag(host_csproj_path,'database')
    def HandleCsprojEventbus(self, service_options, host_csproj_path):
        eventbus_enabled = 'eventbus' in service_options
        
        if(eventbus_enabled):
            eventbus_instance = FindEventBusWithName(self.projectOptions, service_options['eventbus']['provider'])

            eventbus_type = eventbus_instance['type']
            filter_sub_region(host_csproj_path,'eventbus',eventbus_type)
        else:
            filter_region_with_tag(host_csproj_path,'eventbus')

    def HandleCSharpLogging(self, service, sharp_file_path):
        logging_enabled = 'logging' in service
        if logging_enabled:
            if 'type' not in service['logging']:
                logging_type = 'serilog'
            else:
                logging_type = 'microsoft'
        if(logging_enabled):
            filter_sub_region(sharp_file_path,'logging',logging_type)
        else:        
            filter_region_with_tag(sharp_file_path,'logging')
    def HandleCSharpDatabase(self, service_options, sharp_file_path):
        database_enabled = 'database' in service_options
        
        if(database_enabled):
            database_instance = FindDatabaseWithName(self.projectOptions, service_options['database']['provider'])
            database_type = database_instance['type']
            filter_sub_region(sharp_file_path,'database',database_type)
        else:
            filter_region_with_tag(sharp_file_path,'database')
    def HandleCSharpCache(self, service_options, sharp_file_path):
        cache_enabled = 'cache' in service_options
        
        if(cache_enabled):
            cache_type = service_options['cache']['type']
            filter_sub_region(sharp_file_path,'cache',cache_type)
        else:        
            filter_region_with_tag(sharp_file_path,'cache')
    def HandleCSharpServer(self, service_options,sharp_file_path):
        server_enabled = 'server' in service_options
        if(server_enabled):
            server_instance = FindServerWithName(self.projectOptions, service_options['server']['provider'])
            server_type = server_instance['type']
            filter_sub_region(sharp_file_path,'server',server_type)
        else:
            filter_region_with_tag(sharp_file_path,'server')
    def HandleCSharpSwagger(self, dotnet_service, sharp_file_path):
        is_swagger_in_config = 'swagger' in dotnet_service
        if(is_swagger_in_config):
            swagger_enabled = dotnet_service['swagger']
            if not swagger_enabled:
                filter_region_with_tag(sharp_file_path,'swagger')
    def HandleCSharpEventbus(self, service_options, sharp_file_path):
        eventbus_enabled = 'eventbus' in service_options
        
        if(eventbus_enabled):
            eb_replace_dict = {}
            
            eventbus_instance = FindEventBusWithName(self.projectOptions, service_options['eventbus']['provider'])

            if eventbus_instance['type'] == 'rabbitmq':
                eb_replace_dict['{{rabbitmq:host}}'] = 'rabbitmq://'+eventbus_instance['name']
                eb_replace_dict['{{rabbitmq:host-dev}}'] = 'localhost'
                eb_replace_dict['{{rabbitmq:user:username}}'] = 'doom'
                eb_replace_dict['{{rabbitmq:user:password}}'] = 'machine'
                if 'docker_compose_override' in eventbus_instance:
                    if 'envoronment' in eventbus_instance['docker_compose_override']:
                        eb_replace_dict['{{rabbitmq:user:username}}'] = eventbus_instance['docker_compose_override']['environment']['RABBITMQ_DEFAULT_USER']
                        eb_replace_dict['{{rabbitmq:user:password}}'] = eventbus_instance['docker_compose_override']['environment']['RABBITMQ_DEFAULT_PASSWORD']
            replace_template_file(sharp_file_path,eb_replace_dict)
            eventbus_type = eventbus_instance['type']
            filter_sub_region(sharp_file_path,'eventbus',eventbus_type)
        else:
            filter_region_with_tag(sharp_file_path,'eventbus')


    def ReplaceDotnetNameSpaces(self, file_paths, namespace_name, replace_name):
        replace_dict = {}
        replace_dict[namespace_name] = replace_name
        for file in file_paths:
            if os.path.exists(file):
                replace_template_file(file,replace_dict)


import os
import json
def replace_template_file(filepath,replace_dict):
    with open(filepath,'r') as cs_file:
        cs_content = cs_file.read()
    os.remove(filepath)
    for key, value in replace_dict.items():
        cs_content = cs_content.replace(key,value)
    with open(filepath,'w') as cs_file_new:
        cs_file_new.write(cs_content)
def clear_file_region_tags(file):
    try:
        with open(file,'r+') as f:
            filtered = list(Clear_File_Region_Marks(f))
            f.seek(0)
            f.writelines(filtered)
            f.truncate()
    except:
        print (file)
        pass
    
def filter_region_with_tag(file,tag):
    with open(file, 'r+') as f:
        filtered = list(filter_region(f, 'region ('+tag+')', 'end ('+tag+')'))
        f.seek(0)
        f.writelines(filtered)
        f.truncate()
def filter_region(file, start_delete_key, stop_delete_key):
    """
    Given a file handle, generate all lines except those between the specified
    regions.
    """
    lines = iter(file)
    try:
        while True:
            line = next(lines)
            if start_delete_key in line and ':all' not in line:
                # Discard all lines up to and including the stop marker
                while stop_delete_key not in line:
                    line = next(lines)
                line = next(lines)            
            yield line
    except StopIteration:
        return
def filter_sub_region(
    file,
    parent_marker,
    child_marker):
    lines = iter(file)
    parent_marker_start = 'region ('+parent_marker+':'
    parent_marker_end = 'end ('+parent_marker+':'
    try:
        while True:
            line = next(lines)
            if parent_marker_start in line and child_marker not in line:
                while parent_marker_end not in line:
                    line = next(lines)
            yield line
    except StopIteration:
        return
def Clear_File_Region_Marks(file):
    lines = iter(file)
    region_marks = ['//& region','<!-- region','//& end','<!-- end']
    try:
        for line in lines:
            found = False
            for mark in region_marks:
                if mark in line:
                    found = True
            if not found:
                yield line
        while True:
            line = next(lines)
            
    except StopIteration:
        return
def ClearRegionLines(file_paths):
    for file in file_paths:        
        clear_file_region_tags(file)
def RemovePackagesFromJson(file,packages):
    with open(file) as f:
        package_info = json.load(f)
        for package in packages:
            package_info['dependencies'].pop(package, None)
        f.seek(0)
        json.dump(package_info,f)


from microboiler.modules.basemodule import BaseModule
import os
from microboiler.modules.devops.docker import Docker

class RabbitMq(BaseModule):
    def __init__(self, projectOptions, project_templates_paths, outputPath):
        super().__init__(projectOptions, project_templates_paths, outputPath)

    def HandleRabbitMq(self,rabbit_options):
        
        docker_options = self.GetRabbitmqDockerOptions(rabbit_options)
        docker_instance = Docker.getInstance()
        docker_instance.AddService(rabbit_options['name'],docker_options)
        docker_instance.AddVolume('rabbit-volume')

    def GetRabbitmqDockerOptions(self, rabbit_options):
        # No need rabbitmq to know about subs.
        # rabbit_api_services = self.FindApiServicesUsesRabbitmq(rabbit_options['name'])
        # rabbit_identity_services = self.FindIdentityServicesUsesRabbitmq(rabbit_options['name'])

        docker_volume_dir = os.path.normpath(os.path.join(self.outputPath,'docker_volumes','rabbitmq'))
        if not os.path.isdir(docker_volume_dir):
            os.makedirs(docker_volume_dir)
        rabbitmq_docker_options = {
            'image': 'rabbitmq:3-management-alpine',
            'container_name': rabbit_options['name'],
            'volumes': ['rabbit-volume:'+docker_volume_dir],
            'ports': ['15672:15672','5672:5672','5671:5671'], # Management, Publish And Subsucribe Ports
            'environment': {
                'RABBITMQ_DEFAULT_PASS':'machine',
                'RABBITMQ_DEFAULT_USER' : 'doom',
            },
            'healthcheck':{
                'test': 'rabbitmq:healtcheck',
                'interval': '30s',
                'timeout': '10s',
                'retries': 5,
            },
            'networks': ['localnet']
            # 'links':rabbit_identity_services + rabbit_api_services # may be unnecessary
        }
        if 'docker_compose_override' in rabbit_options:
            rabbitmq_docker_options.update(rabbit_options['docker_compose_override'])
            
        return rabbitmq_docker_options
    def FindApiServicesUsesRabbitmq(self, rabbit_name):
        api_services = []
        for service in self.projectOptions['api_services']:
            for key, value in service.items():            
                if 'eventbus' in value:
                    if value['eventbus']['provider'] == rabbit_name:
                        api_services.append(value['name'])
        return api_services

    def FindIdentityServicesUsesRabbitmq(self,rabbit_name):
        i_services = []
        for service in self.projectOptions['identity_services']:
            for key, value in service.items():            
                if 'eventbus' in value:
                    if value['eventbus']['provider'] == rabbit_name:
                        i_services.append(value['name'])
        return i_services

    



import setuptools

setuptools.setup(
    name="microboiler",
    version="0.1.8",
    author="Okan Aslankan",
    author_email="okn.aslnkn@gmail.com",
    description="A Python tool for generating various types of microservice architecture projects.",
    url="https://github.com/DooMachine/microboiler",
    install_requires=['pyyaml==3.12','python-nginx==1.4.1'],
    packages=setuptools.find_packages(),
    scripts=['microboiler/microboiler'],
    include_package_data=True,
    keywords='microservices node .net angular ionic docker nginx devops development',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    project_urls={ 
        'Bug Reports': 'https://github.com/DooMachine/microboiler/issues',
        'Funding': 'https://donate.pypi.org',
        'Source': 'https://github.com/DooMachine/microboiler/',
    },
)